from typing import List
import chronoptics.tof as tof
import matplotlib.pyplot as plt
import numpy as np


def handle_close(evt):
    cam.stop()


def getFrame(frames: List[tof.Data], frame_type: tof.FrameType):
    for frame in frames:
        if frame.frameType() == frame_type:
            return frame
    return None


cam = tof.KeaCamera(serial="202001a")

tof.selectStreams(
    cam, [tof.FrameType.BGR, tof.FrameType.RADIAL, tof.FrameType.INTENSITY]
)

cam.start()

frames = cam.getFrames()

bgr_frame = getFrame(frames, tof.FrameType.BGR)
i_frame = getFrame(frames, tof.FrameType.INTENSITY)
r_frame = getFrame(frames, tof.FrameType.RADIAL)

fig = plt.figure()
fig.canvas.mpl_connect("close_event", handle_close)

ax0 = plt.subplot(131)
ax1 = plt.subplot(132)
ax2 = plt.subplot(133)

ax0.set_title("Colour")
bgrimg = ax0.imshow(
    np.flipud(np.flip(np.array(bgr_frame), axis=2)),
    vmin=0,
    vmax=255,
    interpolation="none",
)

ax1.set_title("Intensity")
iimg = ax1.imshow(
    np.array(i_frame),
    vmin=0,
    vmax=255,
    interpolation="none",
    cmap="gray",
)

ax2.set_title("Distance")
rimg = ax2.imshow(
    np.array(r_frame),
    vmin=0,
    vmax=65535,
    interpolation="none",
    cmap="jet",
)

while cam.isStreaming():
    frames = cam.getFrames()

    # It comes out as a BGR, but matplotlib displays as an RGB.
    bgr_frame = getFrame(frames, tof.FrameType.BGR)
    i_frame = getFrame(frames, tof.FrameType.INTENSITY)
    r_frame = getFrame(frames, tof.FrameType.RADIAL)

    bgrimg.set_data(np.flipud(np.flip(np.array(bgr_frame), axis=2)))
    iimg.set_data(np.array(i_frame))
    rimg.set_data(np.array(r_frame))

    plt.draw()
    plt.show(block=False)

    # This makes sure the event loop is updated
    plt.pause(0.1)
